#include "GtkMultTable.h"
#include "MultPuzzle.h"
#include "CharBox.h"
#include "DigitChooser.h"
#include "config.h"
#include "gettext.h"
#include <ctype.h>

GtkMultTable::GtkMultTable (MultPuzzle *p)
	: Gtk::Table (1, 1, TRUE), puzzle (p)
{
	this->restart (p);
	
	this->set_col_spacings (3);
	this->set_row_spacings (3);
	
	this->set_border_width (12);
	
	puzzle->signal_changed ().connect (sigc::mem_fun (*this, &GtkMultTable::redraw));
}

GtkMultTable::~GtkMultTable (void)
{
}

void GtkMultTable::restart (MultPuzzle *p)
{
	Gtk::HSeparator *sep;
	Gtk::Label *l;
	char *y, *z;
	
	this->puzzle = p;
	
	y = this->puzzle->getMultiplier ();
	z = this->puzzle->getAnswer ();
	
	this->resize (5 + strlen (y), 1 + strlen (z));
	
	free (y);
	free (z);
	
	l = manage (new Gtk::Label ("×", Gtk::ALIGN_CENTER));
	this->attach (*l, 0, 1, 1, 2);
	
	l = manage (new Gtk::Label ("+", Gtk::ALIGN_CENTER));
	this->attach (*l, 0, 1, this->property_n_rows () - 3, 
		this->property_n_rows () - 2);
	
	sep = manage (new Gtk::HSeparator ());
	this->attach (*sep, 0, this->property_n_columns (),
	this->property_n_rows () - 2, this->property_n_rows () - 1);
	
	sep = manage (new Gtk::HSeparator ());
	this->attach (*sep, 0, this->property_n_columns (), 2, 3);
	
	this->redraw ();
}


void GtkMultTable::redraw (void)
{
	char *x, *y, *z;
	
	x = this->puzzle->getMultiplicand ();
	y = this->puzzle->getMultiplier ();
	z = this->puzzle->getAnswer ();
	
	this->addString (x, 0, 0);
	this->addString (y, 1, 0);
	this->addString (z, this->property_n_rows () - 1, 0);
	
	for (int i = 0; i < this->puzzle->getNumAddends (); i++)
	{
		char *n = this->puzzle->getAddend (i);
		this->addString (n, 3 + i, i);
		free (n);
	}
	
	free (x);
	free (y);
	free (z);
	
	this->show_all ();
}

void GtkMultTable::addString (const char *str, int row, int offset)
{
	int size = strlen (str);
	int startCol = this->property_n_columns () - size - offset;
	
	for (int i = 0; i < size; i++)
	{
		CharBox *b;
		
		b = manage (new TableBox (str[i], this->puzzle));
		
		b->set_size_request (36, 24);
		this->attach (*b, startCol + i, startCol + i + 1, row, row + 1,
			Gtk::FILL | Gtk::EXPAND, Gtk::FILL | Gtk::EXPAND, 0, 0);
	}
}


/* table box stuff */
GtkMultTable::TableBox::TableBox (char c, MultPuzzle *p)
	: CharBox (c), puzzle (p)
{
	this->set_events (Gdk::BUTTON_RELEASE_MASK);
}

bool GtkMultTable::TableBox::on_button_release_event (GdkEventButton *e)
{
	if (isalpha (character)) {
		char chosen;
		DigitChooser dc (*((Gtk::Window *) this->get_toplevel ()), *puzzle, character);
		
		chosen = dc.run ();
		
		if (chosen)
			puzzle->guess (chosen, character);
		
		return TRUE;
	}
	else {
		return FALSE;
	}
}
